
/* Chrysalide - Outil d'analyse de fichiers binaires
 * node.c - ligne de contenu Yaml
 *
 * Copyright (C) 2019 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "node.h"


#include <malloc.h>
#include <string.h>


#include "collection.h"



/* Noeud d'une arborescence au format Yaml (instance) */
struct _GYamlNode
{
    GObject parent;                         /* A laisser en premier        */

    GYamlLine *key;                         /* Clef principale du noeud    */
    GYamlCollection *collection;            /* Collection de noeuds        */

};

/* Noeud d'une arborescence au format Yaml (classe) */
struct _GYamlNodeClass
{
    GObjectClass parent;                    /* A laisser en premier        */

};


/* Initialise la classe des noeuds d'arborescence Yaml. */
static void g_yaml_node_class_init(GYamlNodeClass *);

/* Initialise une instance de noeud d'arborescence Yaml. */
static void g_yaml_node_init(GYamlNode *);

/* Supprime toutes les références externes. */
static void g_yaml_node_dispose(GYamlNode *);

/* Procède à la libération totale de la mémoire. */
static void g_yaml_node_finalize(GYamlNode *);



/* Indique le type défini pour un noeud d'arborescence Yaml. */
G_DEFINE_TYPE(GYamlNode, g_yaml_node, G_TYPE_OBJECT);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des noeuds d'arborescence Yaml.         *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_yaml_node_class_init(GYamlNodeClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_yaml_node_dispose;
    object->finalize = (GObjectFinalizeFunc)g_yaml_node_finalize;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node = instance à initialiser.                               *
*                                                                             *
*  Description : Initialise une instance de noeud d'arborescence Yaml.        *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_yaml_node_init(GYamlNode *node)
{
    node->key = NULL;
    node->collection = NULL;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node = instance d'objet GLib à traiter.                      *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_yaml_node_dispose(GYamlNode *node)
{
    g_clear_object(&node->key);

    g_clear_object(&node->collection);

    G_OBJECT_CLASS(g_yaml_node_parent_class)->dispose(G_OBJECT(node));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node = instance d'objet GLib à traiter.                      *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_yaml_node_finalize(GYamlNode *node)
{
    G_OBJECT_CLASS(g_yaml_node_parent_class)->finalize(G_OBJECT(node));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : key = line Yaml représentant la clef du futur noeud.         *
*                                                                             *
*  Description : Construit un noeud d'arborescence Yaml.                      *
*                                                                             *
*  Retour      : Instance mise en place ou NULL en cas d'échec.               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GYamlNode *g_yaml_node_new(GYamlLine *key)
{
    GYamlNode *result;                      /* Structure à retourner       */

    result = g_object_new(G_TYPE_YAML_NODE, NULL);

    /**
     * Le paragraphe "3.2.2.1. Keys Order" des spécifications précise
     * qu'une séquence n'est qu'un noeud sans correspondance clef/valeur.
     *
     * Cette situation doit donc être prise en compte.
     */

    if (key != NULL)
    {
        result->key = key;
        g_object_ref(G_OBJECT(key));
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node = noeud d'arborescence Yaml à consulter.                *
*                                                                             *
*  Description : Fournit la ligne principale associée à un noeud.             *
*                                                                             *
*  Retour      : Ligne Yaml à l'origine du noeud.                             *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GYamlLine *g_yaml_node_get_yaml_line(const GYamlNode *node)
{
    GYamlLine *result;                      /* Ligne d'origine à renvoyer  */

    result = node->key;

    if (result != NULL)
        g_object_ref(G_OBJECT(result));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node   = noeud d'arborescence Yaml à compléter.              *
*                collec = collection de noeuds Yaml.                          *
*                                                                             *
*  Description : Attache une collection de noeuds Yaml à un noeud.            *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_yaml_node_set_collection(GYamlNode *node, GYamlCollection *collec)
{
    g_clear_object(&node->collection);

    g_object_ref_sink(G_OBJECT(collec));
    node->collection = collec;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node = noeud d'arborescence Yaml à consulter.                *
*                                                                             *
*  Description : Fournit une éventuelle collection rattachée à un noeud.      *
*                                                                             *
*  Retour      : Collection de noeuds Yaml ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GYamlCollection *g_yaml_node_get_collection(const GYamlNode *node)
{
    GYamlCollection *result;                /* Collection à renvoyer       */

    result = node->collection;

    if (result != NULL)
        g_object_ref(G_OBJECT(result));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node  = noeud d'arborescence Yaml à consulter.               *
*                path  = chemin d'accès à parcourir.                          *
*                nodes = liste de noeuds avec correspondance établie. [OUT]   *
*                count = quantité de ces noeuds. [OUT]                        *
*                                                                             *
*  Description : Recherche les noeuds correspondant à un chemin.              *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void _g_yaml_node_find_by_path(GYamlNode *node, const char *path, GYamlNode ***nodes, size_t *count)
{
    GYamlLine *line;                        /* Ligne Yaml liée au noeud    */
    const char *key;                        /* Clef associée au noeud      */
    bool matched;                           /* Correspondance établie ?    */
    char *next;                             /* Prochaine partie du chemin  */
    size_t cmplen;                          /* Etendue de la comparaison   */
    int ret;                                /* Bilan d'une comparaison     */
    GYamlCollection *collec;                /* Collection de noeuds        */

    if (path[0] == '/')
        path++;

    /* Correspondance au niveau du noeud ? */

    line = g_yaml_node_get_yaml_line(node);

    if (line != NULL)
    {
        key = g_yaml_line_get_key(line);

        if (path[0] == '\0')
            matched = true;

        else
        {
            next = strchr(path, '/');

            cmplen = (next == NULL ? strlen(path) : next - path);

            if (cmplen == 0)
                goto cont;

            ret = strncmp(path, key, cmplen);

            if (ret != 0)
                goto done;

            else
            {
                if (next == NULL)
                    matched = true;

                else
                {
                    path += cmplen;
                    goto cont;
                }

            }

        }

        if (matched)
        {
            *nodes = realloc(*nodes, ++(*count) * sizeof(GYamlNode **));

            g_object_ref(G_OBJECT(node));
            (*nodes)[*count - 1] = node;

            goto done;

        }

    }

 cont:

    collec = g_yaml_node_get_collection(node);

    if (collec != NULL)
    {
        _g_yaml_collection_find_by_path(collec, path, nodes, count);

        g_object_unref(G_OBJECT(collec));

    }

 done:

    if (line != NULL)
        g_object_unref(G_OBJECT(line));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : node  = noeud d'arborescence Yaml à consulter.               *
*                path  = chemin d'accès à parcourir.                          *
*                nodes = liste de noeuds avec correspondance établie. [OUT]   *
*                count = quantité de ces noeuds. [OUT]                        *
*                                                                             *
*  Description : Recherche les noeuds correspondant à un chemin.              *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_yaml_node_find_by_path(GYamlNode *node, const char *path, GYamlNode ***nodes, size_t *count)
{
    *nodes = NULL;
    *count = 0;

    _g_yaml_node_find_by_path(node, path, nodes, count);

}
